﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Configuration;

namespace DataSyncClient
{
    /**
     * Downloads new or updated donations to a local XML file.
     */
    class DonationDownloader
    {

        private Convio.SoapClient client;
        private Convio.Session session;

        public DonationDownloader(Convio.SoapClient client, Convio.Session session)
        {
            this.client = client;
            this.session = session;
        }

        /**
         * Builds a lookup table of donation levels
         */
        private Dictionary<String, Convio.CrossReferenceValue> getCrossReferences()
        {
            Dictionary<String, Convio.CrossReferenceValue> xrefs = 
                new Dictionary<string, DataSyncClient.Convio.CrossReferenceValue>();
            String query = 
                "select XrefTypeName,ConvioKey,ForeignKey from CrossReferenceValue where XrefTypeId = 1002";

            int page = 1;
            while (true)
            {
                Convio.Record[] records = client.Query(session, query, page.ToString(), "100");
                if (records == null || records.Length == 0)
                {
                    if (page == 1)
                    {
                        System.Console.WriteLine("No cross reference values retrieved");
                    }
                    break;
                }

                foreach (Convio.Record record in records)
                {
                    Convio.CrossReferenceValue xref = (Convio.CrossReferenceValue)record;
                    xrefs[xref.ConvioKey] = xref;
                }

            }

            return xrefs;
        }

        /**
         * Makes a series of requests to retrieve the full set of donations of the specified type.
         * 
         * Adds a cross reference value to each donation if available.
         */
        public void download(DownloadType type)
        {
            String typeLabel = type.ToString().ToLower();
            String partitionId = ConfigurationSettings.AppSettings["partitionId"];

            DataSet downloadDonationsData = new DataSet(type.ToString());
            DataTable downloadDonationsTable = new DataTable("Donation");
            downloadDonationsData.Tables.Add(downloadDonationsTable);
            downloadDonationsTable.Columns.Add(new DataColumn("TransactionId"));
            downloadDonationsTable.Columns.Add(new DataColumn("Amount"));
            downloadDonationsTable.Columns.Add(new DataColumn("CampaignId"));
            downloadDonationsTable.Columns.Add(new DataColumn("FormId"));
            downloadDonationsTable.Columns.Add(new DataColumn("DonationLevelId"));
            downloadDonationsTable.Columns.Add(new DataColumn("CrossReference"));
            downloadDonationsTable.Columns.Add(new DataColumn("ConstituentId"));
            downloadDonationsTable.Columns.Add(new DataColumn("FirstName"));
            downloadDonationsTable.Columns.Add(new DataColumn("LastName"));
            downloadDonationsTable.Columns.Add(new DataColumn("MemberId"));

            Dictionary<String, Convio.CrossReferenceValue> xrefs = getCrossReferences();

            String[] fields = { "TransactionId", "CampaignId", "DonationLevelId", "FormId", 
                                  "Payment.Amount", "Donor.ConsId", "Donor.ConsName", "Donor.MemberId" };
            int page = 1;
            while (true)
            {
                Convio.Record[] records = (type == DownloadType.Inserts) ?
                    client.GetIncrementalInserts(session, partitionId, "Donation", page.ToString(), "100", fields) :
                    client.GetIncrementalUpdates(session, partitionId, "Donation", page.ToString(), "100", fields);
                if (records == null || records.Length == 0)
                {
                    if (page == 1)
                    {
                        System.Console.WriteLine("No Donation " + typeLabel + " to download");
                    }
                    break;
                }
                int start = (page - 1) * 100 + 1;
                System.Console.WriteLine("Downloading Donation " + typeLabel + " " +
                    start + " to " + (start + records.Length));

                foreach (Convio.Record record in records)
                {
                    Convio.Donation Donation = (Convio.Donation)record;
                    DataRow row = downloadDonationsTable.Rows.Add();

                    row["TransactionId"] = Donation.TransactionId;
                    row["ConstituentId"] = Donation.Donor.ConsId;
                    row["FirstName"] = Donation.Donor.ConsName.FirstName;
                    row["LastName"] = Donation.Donor.ConsName.LastName;
                    row["MemberId"] = Donation.Donor.MemberId;
                    row["FormId"] = Donation.FormId;
                    row["CampaignId"] = Donation.CampaignId;
                    row["DonationLevelId"] = Donation.DonationLevelId;
                    row["Amount"] = Donation.Payment.Amount;

                    // set the cross reference value if a match is found in the lookup table
                    String convioKey = Donation.CampaignId + '.' + Donation.DonationLevelId;
                    if (xrefs.ContainsKey(convioKey))
                    {
                        Convio.CrossReferenceValue xref = xrefs[convioKey];
                        row["CrossReference"] = xref.ForeignKey;
                    }
                    else
                    {
                        row["CrossReference"] = "NO_CROSS_REFERENCE";
                    }
                }

                page++;
            }

            String downloadDonationsFile = 
                ConfigurationSettings.AppSettings["downloadDonation" + typeLabel + "File"];
            downloadDonationsData.WriteXml(downloadDonationsFile);
            downloadDonationsData.Dispose();
        }
    }
}
